﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using ProcessSimulationInterfaces;

namespace ProcessCommunication
{
    public class SocketCommunicator
    {
        IMotorProcessSystem processSystem;          // Give you access til MODEL / simulation of physich system

        private string socketIP;
        private IPAddress ip = IPAddress.Any;
        private int port;
        private volatile bool stop = false;
        System.Threading.Thread socketThread;

        public SocketCommunicator(IMotorProcessSystem processSystem,
                                   string socketIP, int port )
        {
            this.processSystem = processSystem;
            if (socketIP != null && socketIP.Length > 0)
                this.ip = IPAddress.Parse(socketIP);
            this.port = port;
        }

        public void StartSocketServiceHandler()
        {
            if (socketThread == null)
            {
                socketThread = new System.Threading.Thread(SocketServicesHandler);
                socketThread.IsBackground = true;
                socketThread.Start();
            }
        }


        private void SocketServicesHandler()
        {
            TcpListener listener = new TcpListener(ip, port);
            listener.Start();

            while (!stop)
            {
                try
                {
                    // wait for client connection 
                    Socket clientSocket = listener.AcceptSocket();

                    // setup input og output streams to communicate text mode
                    NetworkStream netStream = new NetworkStream(clientSocket);
                    StreamWriter writer = new StreamWriter(netStream);
                    StreamReader reader = new StreamReader(netStream);

                    string clientCommand;
                    do
                    {
                        // get command from client
                        string clientCommandText = reader.ReadLine().Trim().ToLower();
                        string[] clientCommandElements = clientCommandText.Split(' ');
                        clientCommand = clientCommandElements[0];
                        switch (clientCommand)
                        {
                            case "setinput":
                                {
                                    string digitalinputValue = clientCommandElements[1]; // argument value
                                    int inputValue = int.Parse(digitalinputValue);
                                    processSystem.SetNewProcessInput(inputValue);
                                    writer.WriteLine("ok " + inputValue);
                                    writer.Flush();
                                }
                                break;
                            case "getinput":
                                {
                                    int inputValue = processSystem.GetProcessInput();
                                    writer.WriteLine("" + inputValue);
                                    writer.Flush();
                                }
                                break;

                            // Conmmunication for other command is to be inserted


                            case "exit":
                                break;
                            default:
                                writer.WriteLine("unknown");
                                writer.Flush();
                                break;

                        }
                    } while (clientCommand != "exit");

                        writer.Close();
                    reader.Close();
                    netStream.Close();
                    clientSocket.Close();

                }
                catch (Exception e)
                {

                }

            }

        }


    }
}
